/*
 * this class is a wrapper of JAMA ( http://math.nist.gov/javanumerics/jama/ )
 */

import pnuts.lang.Numeric;
import Jama.*;
import java.io.*;
import java.text.*;

// It can't extends Matrix because signatures of inverse() are different from Numeric's.

public class PnutsMatrix implements Numeric {
    Matrix matrix;

    public PnutsMatrix(int m, int n) {
	this.matrix = new Matrix(m, n);
    }

    public PnutsMatrix(int m, int n, double s) {
	this.matrix = new Matrix(m, n, s);
    }

    public PnutsMatrix(double[][] A) {
	this.matrix = new Matrix(A);
    }

    public PnutsMatrix(double[][] A, int m, int n) {
	this.matrix = new Matrix(A, m, n);
    }

    public PnutsMatrix(double vals[], int m) {
	this.matrix = new Matrix(vals, m);
    }

    PnutsMatrix(Matrix m) {
	this.matrix = m;
    }

    public static PnutsMatrix random(int m, int n){
	return new PnutsMatrix(Matrix.random(m, n));
    }

    public static PnutsMatrix identity(int m, int n){
	return new PnutsMatrix(Matrix.identity(m, n));
    }

    public Matrix getMatrix(){
	return matrix;
    }

    public Object add(Object o){
	if (o instanceof PnutsMatrix){
 	    Matrix m = ((PnutsMatrix)o).matrix;
	    return new PnutsMatrix(matrix.plus(m));
	}
	throw new RuntimeException();
    }

    public Object subtract(Object o){
	if (o instanceof PnutsMatrix){
 	    Matrix m = ((PnutsMatrix)o).matrix;
	    return new PnutsMatrix(matrix.minus(m));
	}
	throw new RuntimeException();
    }

    public Object multiply(Object o){
	if (o instanceof PnutsMatrix){
 	    Matrix m = ((PnutsMatrix)o).matrix;
	    return new PnutsMatrix(matrix.times(m));
	} else if (o instanceof Number){
	    Number n = (Number)o;
	    return new PnutsMatrix(matrix.times(n.doubleValue()));
	}
	throw new RuntimeException();
    }

    public Object divide(Object o){
	if (o instanceof PnutsMatrix){
 	    Matrix m = ((PnutsMatrix)o).matrix;
	    return new PnutsMatrix(matrix.times(m.inverse()));
	} else if (o instanceof Number){
	    Number n = (Number)o;
	    return new PnutsMatrix(matrix.times(1 / n.doubleValue()));
	}
	throw new RuntimeException();
    }

    public Object inverse(){
	return new PnutsMatrix(matrix.inverse());
    }

    public Object negate(){
	return new PnutsMatrix(matrix.times(-1.0));
    }

    public int compareTo(Object o){
	if (o instanceof PnutsMatrix){
	    int c1 = matrix.getColumnDimension();
	    int r1 = matrix.getRowDimension();
	    Matrix m2 = ((PnutsMatrix)o).matrix;
	    int c2 = m2.getColumnDimension();
	    int r2 = m2.getColumnDimension();
	    if (c1 != c2 || r1 != r2){
		return NOT_EQUAL;
	    }
	    for (int i = 0; i < c1; i++){
		for (int j = 0; j < r1; j++){
		    if (matrix.get(i, j) != m2.get(i, j)){
			return NOT_EQUAL;
		    }
		}
	    }
	    return EQUAL;
	} else {
	    throw new RuntimeException("not supported");
	}
    }

    public double det() {
	return matrix.det();
    }

    public int rank() {
	return matrix.rank();
    }

    public double cond () {
	return matrix.cond();
    }

    public double[][] getArray(){
	return matrix.getArray();
    }

    public double[][] getArrayCopy () {
	return matrix.getArrayCopy();
    }

    public double[] getColumnPackedCopy () {
	return matrix.getColumnPackedCopy();
    }

    public double[] getRowPackedCopy () {
	return matrix.getRowPackedCopy();
    }

    public int getRowDimension () {
	return matrix.getRowDimension();
    }

    public int getColumnDimension () {
	return matrix.getColumnDimension();
    }

    public double get(int i, int j) {
	return matrix.get(i, j);
    }

    public PnutsMatrix getMatrix(int i0, int i1, int j0, int j1) {
	return new PnutsMatrix(matrix.getMatrix(i0, i1, j0, j1));
    }

    public PnutsMatrix getMatrix (int[] r, int[] c) {
	return new PnutsMatrix(matrix.getMatrix(r, c));
    }

    public PnutsMatrix getMatrix (int i0, int i1, int[] c) {
	return new PnutsMatrix(matrix.getMatrix(i0, i1, c));
    }

    public PnutsMatrix getMatrix (int[] r, int j0, int j1) {
	return new PnutsMatrix(matrix.getMatrix(r, j0, j1));
    }

    public void set(int i, int j, double s){
	matrix.set(i, j, s);
    }

    public void setMatrix (int i0, int i1, int j0, int j1, Matrix X) {
	matrix.setMatrix(i0, i1, j0, j1, X);
    }

    public void setMatrix (int[] r, int[] c, Matrix X) {
	matrix.setMatrix(r, c, X);
    }

    public void setMatrix (int[] r, int j0, int j1, Matrix X) {
	matrix.setMatrix(r, j0, j1, X);
    }

    public void setMatrix (int i0, int i1, int[] c, Matrix X) {
	matrix.setMatrix(i0, i1, c, X);
    }

    public PnutsMatrix transpose () {
	return new PnutsMatrix(matrix.transpose());
    }

    public double norm1 () {
	return matrix.norm1();
    }

    public double norm2 () {
	return matrix.norm2();
    }

    public double normInf () {
	return matrix.normInf();
    }

    public double normF () {
	return matrix.normF();
    }

    public PnutsMatrix uminus () {
	return new PnutsMatrix(matrix.uminus());
    }

    public LUDecomposition lu () {
	return matrix.lu();
    }

    public QRDecomposition qr () {
	return matrix.qr();
    }

    public CholeskyDecomposition chol () {
	return matrix.chol();
    }

    public SingularValueDecomposition svd () {
	return matrix.svd();
    }

    public EigenvalueDecomposition eig () {
	return matrix.eig();
    }

    public PnutsMatrix solve(PnutsMatrix B) {
	return new PnutsMatrix(matrix.solve(B.matrix));
    }

    public PnutsMatrix solveTranspose(PnutsMatrix B) {
	return new PnutsMatrix(matrix.solveTranspose(B.matrix));
    }
    
    public Object clone(){
	try {
	    PnutsMatrix m = (PnutsMatrix)super.clone();
	    m.matrix = (Matrix)matrix.clone();
	    return m;
	} catch (Throwable e){
	    throw new InternalError();
	}
    }

    public static PnutsMatrix read(BufferedReader input) throws java.io.IOException {
	return new PnutsMatrix(Matrix.read(input));
    }

    public void print (int w, int d){
	matrix.print(w, d);
    }

    public void print(PrintWriter output, int w, int d) {
	matrix.print(output, w, d);
    }

    public void print (NumberFormat format, int width) {
	matrix.print(format, width);
    }

    public void print(PrintWriter output, NumberFormat format, int width) {
	matrix.print(output, format, width);
    }

    public String toString(int w, int h){
	CharArrayWriter writer = new CharArrayWriter();
	matrix.print(new PrintWriter(writer), w, h);
	return new String(writer.toCharArray());
    }

    public String toString(){
	return toString(10, 6);
    }
}
