/*
 * @(#) Arithmetic.java	1.5 99/03/30
 *
 * Copyright (c) 1997,1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

import java.lang.reflect.Array;

public final class Arithmetic {
    /*
     * operators
     */
    private static final int ADD = 0;
    private static final int SUBTRACT = 1;
    private static final int MULTIPLY = 2;
    private static final int DIVIDE = 3;
    private static final int REMAINDER = 4;
    private static final int GCD = 5;
    private static final int MOD = 6;
    private static final int COMPARE = 7;
    private static final int NEGATE = 10;
    private static final int ABS = 11;
    private static final int AND = 12;
    private static final int OR = 13;
    private static final int XOR = 14;
    private static final int NOT = 15;
    private static final int SHIFT_R = 16;
    private static final int SHIFT_L = 17;
    private static final int SHIFT_A = 18;
    private static final int POW = 19;

    static Integer zero = new Integer(0);

    public final static int NOT_EQUAL = Numeric.NOT_EQUAL;
    public final static int LEFT_IS_BIGGER = Numeric.LEFT_IS_BIGGER;
    public final static int RIGHT_IS_BIGGER = Numeric.RIGHT_IS_BIGGER;
    public final static int EQUAL = Numeric.EQUAL;

    private final static Integer LEFT = new Integer(LEFT_IS_BIGGER);
    private final static Integer RIGHT = new Integer(RIGHT_IS_BIGGER);
    private final static Integer EQ = new Integer(EQUAL);
    private final static Integer NE = new Integer(NOT_EQUAL);

    static Object numeric_object(int operation, Numeric n1, Object n2){
	switch (operation){
	case ADD:
	    return n1.add(n2);
	case SUBTRACT:
	    return n1.subtract(n2);
	case MULTIPLY:
	    return n1.multiply(n2);
	case DIVIDE:
	    return n1.divide(n2);
	case COMPARE:
	    return new Integer(n1.compareTo(n2));
	}
	throw new IllegalArgumentException();
    }

    static Object object_numeric(int operation, Object n1, Numeric n2){
	switch (operation){
	case ADD:
	    return n2.add(n1);
	case SUBTRACT:
	    return add(n2.negate(), n1);
	case MULTIPLY:
	    return n2.multiply(n1);
	case DIVIDE:
	    return multiply(n2.inverse(), n1);
	case COMPARE:
	    int c = ((Numeric)n2).compareTo(n1);
	    if (c == LEFT_IS_BIGGER){
		return RIGHT;
 	    } else if (c == RIGHT_IS_BIGGER){
		return LEFT;
	    } else if (c == EQUAL){
		return EQ;
	    } else {
		return NE;
	    }
	}
	throw new IllegalArgumentException();
    }

    static Object unary(int operator, Object n1){
	if (n1 instanceof Double){
	    double d = Double.NaN;
	    switch (operator){
	      case NEGATE: d = -((Number)n1).doubleValue(); break;
	      case ABS:    d = Math.abs(((Number)n1).doubleValue()); break;
	      default:     throw new IllegalArgumentException();
	    }
	    return new Double(d);
	}
	if (n1 instanceof Float){
	    float d = Float.NaN;
	    switch (operator){
	      case NEGATE: d = -((Number)n1).floatValue(); break;
	      case ABS:    d = Math.abs(((Number)n1).floatValue()); break;
 	      default:     throw new IllegalArgumentException();
	    }
	    return new Float(d);
	}
	if (n1 instanceof Long ||
	    n1 instanceof Integer ||
	    n1 instanceof Short ||
	    n1 instanceof Byte){
	    long d = 0;
	    switch (operator){
	      case NEGATE: d = -((Number)n1).longValue(); break;
	      case ABS:    d = Math.abs(((Number)n1).longValue()); break;
	      case NOT:    d = ~((Number)n1).longValue(); break;
 	      default:     throw new IllegalArgumentException();
	    }
	    return longOrInt(d);
	}
	if (n1 instanceof Numeric){
	    switch (operator){
	    case NEGATE:
		if (((Numeric)n1).compareTo(zero) == RIGHT_IS_BIGGER){
		    return ((Numeric)n1).negate();
		} else {
		    return n1;
		}
	    }
	}
	throw new IllegalArgumentException();
    }

    static Object binary(int operator, Object n1, Object n2){
	boolean i1 = false;
	boolean i2 = false;
	boolean l1 = false;
	boolean l2 = false;
	boolean bd1 = false;
	boolean bd2 = false;
	boolean bi1 = false;
	boolean bi2 = false;
	boolean f1 = false;
	boolean f2 = false;
	boolean d1 = false;
	boolean d2 = false;

	if (n1 instanceof Integer || n1 instanceof Short || n1 instanceof Byte){
	    i1 = true;
	} else if (n1 instanceof Long){
	    l1 = true;
	} else if (n1 instanceof Float){
	    f1 = true;
	} else if (n1 instanceof Double){
	    d1 = true;
	} else if (n1 instanceof String){
	    if (operator == ADD){
		return concat(n1, n2);
	    }
	} else if (n1 instanceof Numeric){
	    return numeric_object(operator, (Numeric)n1, n2);
	}
	if (n2 instanceof Integer || n2 instanceof Short || n2 instanceof Byte){
	    i2 = true;
	} else if (n2 instanceof Long){
	    l2 = true;
	} else if (n2 instanceof Float){
	    f2 = true;
	} else if (n2 instanceof Double){
	    d2 = true;
	} else if (n2 instanceof String){
	    if (operator == ADD){
		return concat(n1, n2);
	    }
	} else if (n2 instanceof Numeric){
	    return object_numeric(operator, n1, (Numeric)n2);
	}
	if (i1 && i2){
	    return op2_int(operator, ((Number)n1).intValue(), ((Number)n2).intValue());
	}
	if ((i1 || l1) && (i2 || l2)){
	    return op2_long(operator, ((Number)n1).longValue(), ((Number)n2).longValue());
	} else if (d1 || d2){
 	    return op2_double(operator, ((Number)n1).doubleValue(), ((Number)n2).doubleValue());
	} else if (f1 || f2){
	    return op2_float(operator, ((Number)n1).floatValue(), ((Number)n2).floatValue());
	} else if (n1.getClass().isArray() && n2.getClass().isArray()){
	    if (operator == ADD){
		return append(n1, n2);
	    }
	}
	throw new IllegalArgumentException();
    }

    static Object op2_double(int operator, double d1, double d2){
	switch (operator){
	case ADD:
	    return new Double(d1 + d2);
	case SUBTRACT:
	    return new Double(d1 - d2);
	case MULTIPLY:
	    return new Double(d1 * d2);
	case DIVIDE:
	    return new Double(d1 / d2);
	case COMPARE:
	    if (d1 > d2){
		return LEFT;
	    } else if (d1 < d2){
		return RIGHT;
	    } else if (d1 == d2){
		return EQ;
	    } else {
		return NE;
	    }
	case POW:
	    return new Double(Math.pow(d1, d2));
	default:
	    throw new IllegalArgumentException();
	}
    }

    static Object op2_float(int operator, float d1, float d2){
	switch (operator){
	case ADD:
	    return new Float(d1 + d2);
	case SUBTRACT:
	    return new Float(d1 - d2);
	case MULTIPLY:
	    return new Float(d1 * d2);
	case DIVIDE:
	    return new Float(d1 / d2);
	case COMPARE:
	    if (d1 > d2){
		return LEFT;
	    } else if (d1 < d2){
		return RIGHT;
	    } else if (d1 == d2){
		return EQ;
	    } else {
		return NE;
	    }
	case POW:
	    return new Float((float)Math.pow(d1, d2));
	default:
	    throw new IllegalArgumentException();
	}
    }

    static Object op2_long(int operator, long d1, long d2){
	long d3;
	switch (operator){
	case ADD:
	    d3 = d1 + d2;
	    if (d3 > d1 && d2 < 0 || d3 < d1 && d2 > 0){
		return new Double((double)d1 + (double)d2);
	    }
	    return longOrInt(d3);
	case SUBTRACT:
	    d3 = d1 - d2;
	    if (d3 > d1 && d2 > 0 || d3 < d1 && d2 < 0){
		return new Double((double)d1 - (double)d2);
	    }
	    return longOrInt(d3);
	case MULTIPLY:
	    d3 = d1 * d2;
	    if (d2 == 0){
		return zero;
	    } else if (d3 / d2 == d1){
		return longOrInt(d3);
	    } else {
		return new Double((double)d1 * (double)d2);
	    }
	case DIVIDE:
	    d3 = d1 / d2;
	    return longOrInt(d3);
	case MOD:
	    d3 = d1 % d2;
	    return longOrInt(d3);
	case COMPARE:
	    if (d1 > d2){
		return LEFT;
	    } else if (d1 < d2){
		return RIGHT;
	    } else if (d1 == d2){
	        return EQ;
	    } else {
		return NE;
	    }
	case AND:
	    return longOrInt(d1 & d2);
	case OR:
	    return longOrInt(d1 | d2);
	case XOR:
	    return longOrInt(d1 ^ d2);
	case SHIFT_L:
	    int m = 0;
	    long dd = d1;
	    while (dd > 0){
		dd >>= 1;
		m++;
	    }
	    if (d2 + m > 63){
		throw new ArithmeticException("overflow");
	    }
	    return longOrInt(d1 << d2);
	case SHIFT_R:
	    return longOrInt(d1 >> d2);
	case SHIFT_A:
	    return longOrInt(d1 >>> d2);
	case GCD:
	    return longOrInt(gcd(d1, d2));
	case POW:
	    return new Double(Math.pow((double)d1, (double)d2));
	default:
	    throw new IllegalArgumentException();
	}
    }

    static Object op2_int(int operator, int d1, int d2){
	switch (operator){
	  case ADD:
	    return longOrInt((long)d1 + d2);
	  case SUBTRACT:
	    return longOrInt((long)d1 - d2);
	  case MULTIPLY:
	    return longOrInt((long)d1 * d2);
	  case DIVIDE:
	    return longOrInt(d1 / d2);
	  case MOD:
	    return longOrInt(d1 % d2);
	  case COMPARE:
	    if (d1 > d2){
		return LEFT;
	    } else if (d1 < d2){
		return RIGHT;
	    } else if (d1 == d2){
	        return EQ;
	    } else {
	        return NE;
	    }
	  case AND:
	    return longOrInt(d1 & d2);
	  case OR:
	    return longOrInt(d1 | d2);
	  case XOR:
	    return longOrInt(d1 ^ d2);
	  case SHIFT_L:
	    int m = 0;
	    long dd = (long)d1;
	    while (dd > 0){
		dd >>= 1;
		m++;
	    }
	    if (d2 + m > 63){
		throw new ArithmeticException("overflow");
	    }
	    return longOrInt((long)d1 << d2);
	  case SHIFT_R:
	    return longOrInt(d1 >> d2);
	  case SHIFT_A:
	    return longOrInt(d1 >>> d2);
	  case GCD:
	    return new Integer((int)gcd(d1, d2));
	  case POW:
	      return new Double(Math.pow(d1, d2));
	  default:
	    throw new IllegalArgumentException();
	}
    }

    static Number longOrInt(long l){
	if (l <= Integer.MAX_VALUE && l >= Integer.MIN_VALUE){
	    return new Integer((int)l);
	} else {
	    return new Long(l);
    	}
    }

    public final static Object add(Object n1, Object n2){
	return binary(ADD, n1, n2);
    }

    public final static Object subtract(Object n1, Object n2){
	return binary(SUBTRACT, n1, n2);
    }

    public final static Object multiply(Object n1, Object n2){
	return binary(MULTIPLY, n1, n2);
    }

    public final static Object divide(Object n1, Object n2){
	return binary(DIVIDE, n1, n2);
    }

    public final static int compareTo(Object n1, Object n2){
	if ((n1 instanceof Number || n1 instanceof Numeric) &&
	    (n2 instanceof Number || n2 instanceof Numeric)){
	    return ((Number)binary(COMPARE, n1, n2)).intValue();
	} else if (n1 instanceof String && n2 instanceof String){
	    int c = ((String)n1).compareTo((String)n2);
	    if (c < 0){
		return RIGHT_IS_BIGGER;
	    } else if (c > 0){
		return LEFT_IS_BIGGER;
	    } else {
		return EQUAL;
	    }
	} else if (n1 == null){
	    return n2 == null ? EQUAL : NOT_EQUAL;
	} else if (n2 == null){
	    return NOT_EQUAL;
	} else if (n1.getClass().isArray() && n2.getClass().isArray()){
	    int l1 = Array.getLength(n1);
	    int l2 = Array.getLength(n2);
	    if (l2 == l1){
		for (int i = 0; i < l1; i++){
		    if (compareTo(Array.get(n2, i), Array.get(n1, i)) != EQUAL){
			return NOT_EQUAL;
		    }
		}
	    } else {
		return NOT_EQUAL;
	    }
	    return EQUAL;
	} else {
	    return n2.equals(n1) ? EQUAL : NOT_EQUAL;
	}
    }

    public final static Object remainder(Object n1, Object n2){
	return binary(REMAINDER, n1, n2);
    }

    public final static Object gcd(Object n1, Object n2){
	return binary(GCD, n1, n2);
    }

    public final static Object pow(Object n1, Object n2){
	return binary(POW, n1, n2);
    }

    public final static Object mod(Object n1, Object n2){
	return binary(MOD, n1, n2);
    }

    public final static Object and(Object n1, Object n2){
	return binary(AND, n1, n2);
    }

    public final static Object or(Object n1, Object n2){
	return binary(OR, n1, n2);
    }

    public final static Object xor(Object n1, Object n2){
	return binary(XOR, n1, n2);
    }

    public final static Object shiftLeft(Object n1, Object n2){
	return binary(SHIFT_L, n1, n2);
    }

    public final static Object shiftRight(Object n1, Object n2){
	return binary(SHIFT_R, n1, n2);
    }

    public final static Object shiftArithmetic(Object n1, Object n2){
	return binary(SHIFT_A, n1, n2);
    }

    public final static Object abs(Object n1){
	return unary(ABS, n1);
    }

    public static Object negate(Object n1){
	if (n1 instanceof Numeric){
	    return ((Numeric)n1).negate();
	} else {
	    return unary(NEGATE, n1);
	}
    }

    public static Object inverse(Object n1){
	if (n1 instanceof Numeric){
	    return ((Numeric)n1).inverse();
	} else {
	    return divide(new Double(1.0), (Number)n1);
	}
    }

    public final static Object not(Object n1){
	return unary(NOT, n1);
    }

    final static Number decimalNumber(String s, int radix){
	return decimal(s, radix);
    }

    static Number decimal(String s, int radix){
	if (radix == 16){
	    return Long.valueOf(s, radix);
	} else {
	    return Double.valueOf(s);
	}
    }

    static long gcd(long a, long b){ // Euclid's algorithm
	if (b > a){
	    long tmp = a;
	    a = b;
	    b = tmp;
	}
	for(;;){
	    if (b == 0){
		return a;
	    } else if (b == 1){
		return b;
	    } else {
		long tmp = b;
		b = a % b;
		a = tmp;
	    }
	}
    }

    final static String NULL = "null";

    static String concat(Object o1, Object o2){
	String s2;
	if (o2 == null){
	    s2 = NULL;
	} else if (o2.getClass().isArray()){
	    s2 = Pnuts.format(o2);
	} else {
	    s2 = o2.toString();
	}
	String s1;
	if (o1 == null){
	    s1 = NULL;
	} else if (o1.getClass().isArray()){
	    s1 = Pnuts.format(o1);
	} else {
	    s1 = o1.toString();
	}
	return s1 + s2;
    }

    static Object append(Object p1, Object p2){
	int l1 = Array.getLength(p1);
	int l2 = Array.getLength(p2);
	Object p3 = Array.newInstance(p1.getClass().getComponentType(), l1 + l2);
	try {
	    for (int i = 0; i < l1; i++){
		Array.set(p3, i, Array.get(p1, i));
	    }
	    for (int i = 0; i < l2; i++){
		Array.set(p3, i + l1, Array.get(p2, i));
	    }
	    return p3;
	} catch (Exception e){
	    throw new IllegalArgumentException();
	}
    }
}
