/*
 * @(#) DialogOutputStream.java	1.5 99/04/12
 *
 * Copyright (c) 1997,1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

package pnuts.awt;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 * Output stream to a Dialog.
 * The flush() method brings up the dialog.
 *
 * @version	1.5
 */
public class DialogOutputStream extends ByteArrayOutputStream implements ActionListener, KeyListener {

    Frame parent;
    Dialog dialog;
    TextArea textArea;
    Button button;
    private static FlowLayout flowLayout = new FlowLayout();

    public DialogOutputStream(Frame parent) {
	this(parent, 32);
    }

    public DialogOutputStream(Frame parent, int size) {
	this(parent, size, false);
    }

    public DialogOutputStream(Frame parent, int size, boolean modal) {
	super(size);
	this.parent = parent;
	dialog = new Dialog(parent, "Error", modal);
	textArea = new TextArea("", 0, 0, TextArea.SCROLLBARS_BOTH);
	textArea.setEditable(false);
	textArea.setSize(500, 150);
	textArea.setBackground(Color.white);
	textArea.addKeyListener(this);
	dialog.add("Center", textArea);

	button = new Button("OK");
	button.addActionListener(this);

	Panel panel = new Panel();
	panel.setLayout(flowLayout);
	panel.add(button);
	dialog.add("South", panel);
	dialog.setSize(500, 200);
    }

    public void keyTyped(KeyEvent e){
	dialog.setVisible(false);
	reset();
	textArea.setText("");
    }

    public void keyPressed(KeyEvent e){}
    public void keyReleased(KeyEvent e){}

    public void actionPerformed(ActionEvent e){
	dialog.setVisible(false);
	reset();
	textArea.setText("");
    }

    public Dimension getSize(){
	return dialog.getSize();
    }

    public void setSize(int w, int h){
	dialog.setSize(w, h);
    }

    public void toFront(){
	dialog.toFront();
    }

    public void flush(){
	textArea.setText(new String(toByteArray()));
	Point loc = parent.getLocation();
	Dimension sz1 = parent.getSize();
	Dimension sz2 = dialog.getSize();
	int x = loc.x + (sz1.width - sz2.width) / 2;
	int y = loc.y + (sz1.height - sz2.height) / 2;
	if (x < 0 || y < 0){
	    Dimension d = dialog.getToolkit().getScreenSize();
	    x = (d.width - sz2.width) / 2;
	    y = (d.height - sz2.height) / 2;
	}
	dialog.setLocation(x, y);

	dialog.setVisible(true);
	dialog.toFront();
    }
}
