/*
 * @(#) ProtectedPackage.java	1.3 99/06/03
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.ext;

import java.util.Hashtable;
import pnuts.lang.Pnuts;
import pnuts.lang.Value;
import pnuts.lang.Package;
import pnuts.lang.PackageFactory;
import pnuts.lang.Context;

/**
 * This class is a variation of pnuts.lang.Package class.
 * Only exported symbols are visible from other package.
 * e.g.<pre>
 * import("pnuts.ext.ProtectedPackage")
 * package(ProtectedPackage("foo"))
 * x = 100
 * y = 100
 * export("x")
 * package("")
 * foo::x  ==> 100
 * foo::y  ==> not found
 * </pre>
 * @see	<a href="../../../doc/protected.html">User's Guide</a>
 * @version	1.3
 */
public class ProtectedPackage extends Package {

    /**
     * @serial
     */
    protected Hashtable exports;

    /**
     * make this class the default class of packages
     */
    public static void setDefault(){
	Package.setPackageFactory(new Factory());
    }

    protected ProtectedPackage(){
	super();
    }

    public ProtectedPackage(String name){
	super(name);
    }

    public void export(String symbol, Context context){
	if (context.getCurrentPackage() == this){
	    export(symbol);
	} else {
	    throw new IllegalAccessError(name + "::" + symbol);
	}
    }

    private void export(String symbol){
	exports.put(symbol, symbol);
    }

    final static String[] globals = {
	"import", "package", "load", "loadFile", "eval",
	"getContext", "defined", "quit", "catch", "throw",
	"pnuts_version", "pnuts_load_path",
	"int", "byte", "short", "char", "long", "double", "float", "boolean",
    };

    /**
     * This method is called by the constructors.
     */
    protected void init(){
	exports = new Hashtable();
	if ("".equals(name)){
	    for (int i = 0; i < globals.length; i++){
		export(globals[i]);
	    }
	}
	super.init();
    }

    /**
     * This method is called when this package become the current package.
     * Function export(symbol) is defined for convenience.
     */
    protected void init(Context context){
	super.init(context);
	Pnuts.eval("function export(symbol) package().export(symbol, getContext())", context);
    }

    protected Value lookup(String symbol, Context context){
	if (context != null && context.getCurrentPackage() == this || exports.get(symbol) != null){
	    return super.lookup(symbol, context);
	} else {
	    return null;
	}
    }

    /**
     * Set a value of a symbol in the package
     * @param	symbol a name of variable
     * @param	val the value of the variable
     * @param	context the context
     * @since	Pnuts 1.0beta3
     */
    public void set(String symbol, Object val, Context context){
	if (context != null && context.getCurrentPackage() == this ||
	    exports.get(symbol) != null){
	    super.set(symbol, val, context);
	} else {
	    System.out.println("this = " + this);
	    System.out.println("currentPackage = " + context.getCurrentPackage());

	    throw new IllegalAccessError(name + "::" + symbol);
	}
    }

    /**
     * Delete a symbol from the package
     * @param	symbol a name of variable to be deleted
     * @param	context the context
     */
    public void clear(String symbol, Context context){
	if (context != null && context.getCurrentPackage() == this || exports.get(symbol) != null){
	    super.clear(symbol, context);
	}
    }

    public static class Factory implements PackageFactory {
	public Package createPackage(String name){
	    if (name == null){
		return new ProtectedPackage();
	    } else {
		return new ProtectedPackage(name);
	    }
	}
    }
}
