/*
 * @(#) Prototype.java	1.5 99/03/30
 *
 * Copyright (c) 1997,1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.ext;

import java.util.Hashtable;
import java.io.Serializable;
import pnuts.lang.AbstractData;
import pnuts.lang.PnutsFunction;
import pnuts.lang.Context;

/**
 * An implementation of prototype-based Inheritance, which is simillar to that of ECMAScript.
 * See <a href="/pnuts/doc/proto-oo.html">Pnuts User's Guide</a> for details.
 *
 * @see		pnuts.lang.AbstractData
 * @version	1.5
 * @author	Toyokazu Tomatsu
 */

public class Prototype extends PnutsFunction implements AbstractData, Cloneable, Serializable {
    /**
     * @serial
     */
    Prototype prototype;

    /**
     * @serial
     */
    Hashtable table = new Hashtable(4);

    /**
     * @serial
     */
    String name;

    public Prototype(){
	this("");
    }

    public Prototype(String name){
	this.name = name;
    }

    public Object get(String name){
	if ("prototype".equals(name)){
	    return prototype;
	}
	Object v = table.get(name);
	if (v == null){
	    if (prototype != null){
		return prototype.get(name);
	    }
	}
	return v;
    }

    public void set(String name, Object value){
	if ("prototype".equals(name)){
	    prototype = (Prototype)value;
	    return;
	}
	Object v = table.get(name);
	if (v == null){
	    if (prototype != null){
		prototype.set(name, value);
		return;
	    }
	}
	if (value != null){
	    table.put(name, value);
	} else {
	    table.remove(name);
	}
    }

    protected Object exec(Object args[], Context context){
	if (args.length == 0){
	    return clone();
	} else {
	    throw new RuntimeException("no such constructor");
	}
    }

    public Object invoke(String name, Object[] args){
	if ("clone".equals(name)){
	    return clone();
	}
	PnutsFunction f = (PnutsFunction)get(name);
	if (f != null){
	    PnutsFunction f2 = (PnutsFunction)f.call(new Object[]{this});
	    if (f2 != null){
		return f2.call(args);
	    }
	}
	throw new RuntimeException("no such function: " + name);
    }

    public String toString(){
	return "prototype " + name;
    }

    public Object clone(){
	try {
	    Prototype p = (Prototype)super.clone();
	    p.table = (Hashtable)table.clone();
	    return p;
	} catch (Throwable t){
	    throw new Error("");
	}
    }
}
