/*
 * @(#) ProxyClassLoader.java	1.1 99/03/26
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.ext;

import java.io.InputStream;
import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLClassLoader;
import java.security.CodeSource;

/**
 * This class is used by pnuts.ext.PnutsClient to define the PROXY_CLASS
 * for each codebase.
 *
 * @version	1.1
 * @see		pnuts.ext.PnutsClient
 * @see		pnuts.ext.IPnutsProxy
 * @see		pnuts.ext.PnutsProxy
 */
class ProxyClassLoader extends URLClassLoader {

    private static String PROXY_CLASS = "pnuts.ext.PnutsProxy".intern();

    private CodeSource codesource;

    public ProxyClassLoader(URL base){
	super(new URL[]{base});
	this.codesource = new CodeSource(base, null);
    }

    byte[] getBytecode(String name) throws IOException {
	final String path = "/" + name.replace('.', '/') + ".class";
	URL u = ProxyClassLoader.this.getClass().getResource(path);

	InputStream in = null;
	try {
	    URLConnection con = u.openConnection();
	    in = u.openStream();
	    int len = con.getContentLength();

	    if (len != -1) {
		byte[] b = new byte[len];
		while (len > 0) {
		    int n = in.read(b, b.length - len, len);
		    if (n == -1) {
			throw new IOException("unexpected EOF");
		    }
		    len -= n;
		}
		return b;
	    } else {
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		int total = 0;
		byte[] b = new byte[1024];
		while ((len = in.read(b, 0, b.length)) != -1) {
		    bout.write(b, 0, len);
		}
		return bout.toByteArray();
	    }
	} finally {
	    if (in != null){
		in.close();
	    }
	}
    }

    Class loadProxyClass(String name) throws ClassNotFoundException {
	byte code[] = null;
	try {
	    code = getBytecode(name);
	} catch (IOException e){
	}
	if (code != null){
	    return defineClass(name, code, 0, code.length, codesource);
	} else {
	    throw new ClassNotFoundException(name);
	}
    }

    public Class getPnutsProxy() throws ClassNotFoundException {
	return loadClass(PROXY_CLASS, true);
    }

    public Class loadClass(String name, boolean resolve) throws ClassNotFoundException {
	Class c = findLoadedClass(name);
	if (c == null){
	    name = name.intern();
	    if (PROXY_CLASS == name){
		c = loadProxyClass(PROXY_CLASS);
		if (resolve){
		    resolveClass(c);
		}
		return c;
	    }
	    c = super.loadClass(name, resolve);
	}
	if (resolve){
	    resolveClass(c);
	}
	return c;
    }
}
