/*
 * @(#) UnitSystem.java	1.2 99/03/30
 *
 * Copyright (c) 1997,1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.ext;

import pnuts.lang.Pnuts;
import pnuts.lang.QuantityFactory;
import java.io.Serializable;

/**
 * This class defines a unit system in Pnuts.
 * When an identifier follows a decimal number, the identifier should
 * be one of the units symbols defined.
 * <pre>
 * UnitSystem(["cm", "m"], [1, 100])
 * 1cm + 1m  ==> 101.000cm
 * </pre>
 */
public class UnitSystem implements QuantityFactory, Serializable {

    private static Integer zero = new Integer(0);

    /**
     * @serial
     */
    String units[];

    /**
     * @serial
     */
    Number scale[];

    /**
     * @serial
     */
    int prec;

    public UnitSystem(String units[], Number scale[]){
	init(units, scale);
    }

    void init(String units[], Number scale[]){
	this.units = units;
	this.scale = scale;
	this.prec = 3;
	init();
    }

    public void init(){
	for (int i = 0; i < units.length; i++){
	    Pnuts.registerQuantityFactory(units[i], this);
	}
    }

    public void setPrecision(int prec){
	this.prec = prec;
    }

    int size(){
	return units.length;
    }

    Number getScale(String unit){
	for (int i = 0; i < units.length; i++){
	    if (units[i].equals(unit)){
		return scale[i];
	    }
	}
	throw new RuntimeException("undefined unit name: " + unit);
    }

    public Object make(Number n, String unit){
	return Quantity.make(this, n, unit);
    }
}
