/*
 * @(#) Units.java	1.1 98/06/26
 *
 * Copyright (c) 1997,1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.ext;

import pnuts.lang.Arithmetic;
import java.io.Serializable;
import java.math.BigDecimal;

class Units implements Cloneable, Serializable {
    static int INITIAL_SIZE = 4;
    int dimension[] = new int[INITIAL_SIZE];
    String unitName[] = new String[INITIAL_SIZE];
    UnitSystem system[] = new UnitSystem[INITIAL_SIZE];
    int count = 0;

    public Number relativeScale(Units u){
	Number scale = new Integer(1);
	int n = size();
	for (int i = 0; i < n; i++){
	    Number d1 = system[i].getScale(unitName[i]);
	    int p1 = system[i].prec;
	    int idx = u.getIndex(system[i]);
	    if (idx >= 0){
		Number d2 = u.system[idx].getScale(u.unitName[idx]);
		int p2 = u.system[idx].prec;
		if (p1 < p2){
		    p2 = p1;
		}
		if (dimension[i] == 1){
		    scale = (Number)Arithmetic.multiply(scale, divide(d2, d1, p2));
		} else if (dimension[i] != 0){
		    scale =
			 (Number)Arithmetic.multiply(scale,
						     Arithmetic.pow(divide(d2, d1, p2),
								    new Integer(dimension[i])));
		}
	    }
	}
	return scale;
    }

    protected Object clone(){
	try {
	    Units newUnits =  (Units)super.clone();
	    newUnits.dimension = new int[dimension.length];
	    newUnits.unitName = new String[unitName.length];
	    newUnits.system = new UnitSystem[system.length];
	    System.arraycopy(dimension, 0, newUnits.dimension, 0, dimension.length);
	    System.arraycopy(unitName, 0, newUnits.unitName, 0, unitName.length);
	    System.arraycopy(system, 0, newUnits.system, 0, system.length);
	    return newUnits;
	} catch (Exception e){}
	return null;
    }

    public Units inverse(){
	Units u = (Units)clone();
	int n = size();
	for (int i = 0; i < n; i++){
	    u.dimension[i] = -dimension[i];
	}
	return u;
    }

    Units op(Units units, boolean div){
	Units u = (Units)clone();
	int n = units.size();
	for (int i = 0; i < n; i++){
	    String s = units.unitName[i];
	    int d = units.dimension[i];
	    if (div){
		d = -d;
	    }
	    UnitSystem sys = units.system[i];
	    u.addDimension(sys, s, d);
	}
	return u;
    }

    public Units multiply(Units units){
	return op(units, false);
    }

    public Units divide(Units units){
	return op(units, true);
    }

    static Number divide(Number n1, Number n2, int prec){
	BigDecimal d = new BigDecimal(n1.doubleValue() / n2.doubleValue());
	return d.setScale(prec, BigDecimal.ROUND_HALF_UP);
    }

    public boolean equals(Object obj){
	if (obj instanceof Units){
	    return equals((Units)obj);
	}
	return false;
    }

    int size(){
	return count;
    }

    boolean equals(Units units){
	int n = size();
	if (units.size() != n){
	    return false;
	}
	for (int i = 0; i < n; i++){
	    UnitSystem s = units.system[i];
	    int d = units.dimension[i];
	    if (!check(s, d)){
		return false;
	    }
	}
	return true;
    }

    void addDimension(UnitSystem s, String u, int d){
	int n = size();
	for (int i = 0; i < n; i++){
	    if (s == system[i]){
		dimension[i] += d;
		return;
	    }
	}
	add(s, u, d);
    }

    void add(UnitSystem sys, String u, int d){
	ensure(count + 1);
	system[count] = sys;
	unitName[count] = u;
	dimension[count] = d;
	++count;
    }

    void ensure(int n){
	int size = system.length;
	if (size < n){
	    int newSize = size * 2;
	    int newDimension[] = new int[newSize];
	    String newUnitName[] = new String[newSize];
	    UnitSystem newSystem[] = new UnitSystem[newSize];
	    System.arraycopy(dimension, 0, newDimension, 0, size);
	    System.arraycopy(unitName, 0, newUnitName, 0, size);
	    System.arraycopy(system, 0, newSystem, 0, size);
	    dimension = newDimension;
	    unitName = newUnitName;
	    system = newSystem;
	}
    }

    int getIndex(UnitSystem s){
	int n = size();
	for (int i = 0; i < n; i++){
	    if (s == system[i]){
		return i;
	    }
	}
	return -1;
    }

    boolean check(UnitSystem s, int d){
	int i = getIndex(s);
	return i >= 0 && dimension[i] == d;
    }

    public String toString(){
	StringBuffer s = new StringBuffer();
	int n = size();
	if (n < 1){
	    return "";
	}
	int d = dimension[0];
	if (d != 0){
	    s.append(unitName[0]);
	    if (d != 1){
		s.append('^');
		s.append(String.valueOf(d));
	    }
	}
	for (int i = 1; i < n; i++){
	    d = dimension[i];
	    if (d != 0){
		s.append('*');
		s.append(unitName[i]);
		if (d != 1){
		    s.append(String.valueOf(d));
		}
	    }
	}
	return s.toString();
    }
}
