/*
 * %Z% %M%	%I% %E%
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.ext;

import pnuts.lang.Visitor;
import pnuts.lang.PnutsParserTreeConstants;
import pnuts.lang.SimpleNode;
import pnuts.lang.Context;

public class UnparseVisitor implements Visitor, PnutsParserTreeConstants {

    boolean escape = false;

    public UnparseVisitor(){
    }

    public UnparseVisitor(boolean escape){
	this.escape = escape;
    }

    String operand(SimpleNode node, int idx){
	int id = node.jjtGetChild(idx).id;
	if (id == JJTIDNODE ||
	    id == JJTINDEXNODE ||
	    id == JJTSTATICMETHODNODE ||
	    id == JJTSTATICMEMBERNODE ||
	    id == JJTAPPLICATIONNODE ||
	    id == JJTMETHODNODE ||
	    id == JJTMEMBERNODE ||
	    id == JJTCASTEXPRESSION ||
	    id == JJTINTEGERNODE ||
	    id == JJTFLOATINGNODE ||
	    id == JJTCHARACTERNODE ||
	    id == JJTSTRINGNODE ||
	    id == JJTTRUENODE ||
	    id == JJTFALSENODE ||
	    id == JJTNULLNODE){
	    return accept(node, idx);
	} else {
	    return "(" + accept(node, idx) + ")";
	}
    }

    String binary(SimpleNode node, String operator){
	String a1 = operand(node, 0);
	String a2 = operand(node, 1);
	return a1 + " " + operator + " " + a2;
    }

    public Object startSet(SimpleNode node, Context context){
	return expressionList(node, context, '\n');
    }

    public Object start(SimpleNode node, Context context){
	return node.jjtGetChild(0).accept(this, context);
    }

    public Object expressionList(SimpleNode node, Context context){
	return expressionList(node, context, ';');
    }

    public Object expressionList(SimpleNode node, Context context, char delimiter){
	int num = node.jjtGetNumChildren();
	StringBuffer buf = new StringBuffer();
	if (num > 0){
	    buf.append((String)accept(node, 0));
	    for (int i = 1; i < num; i++){
		buf.append(delimiter);
		buf.append((String)accept(node, i));
	    }
	}
	return buf.toString();
    }

    public Object idNode(SimpleNode node, Context context){
	int dim = node.jjtGetNumChildren();
	String s = node.str;
	for (int i = 0; i < dim; i++){
	    s += "[]";
	}
	return s;
    }

    public Object global(SimpleNode node, Context context){
	return "::" + node.str;
    }

    public Object arrayType(SimpleNode node, Context context){
	return accept(node, 0) + "[]";
    }

    public Object listElements(SimpleNode node, Context context){
	int n = node.jjtGetNumChildren();
	String s = "[";
	if (n > 0){
	    s += accept(node, 0);
	    for (int i = 1; i < n; i++){
		s += ", " + accept(node, i);
	    }
	}
	return s + "]";
    }

    public Object castExpression(SimpleNode node, Context context){
	int n = node.jjtGetNumChildren();
	String type = accept(node, 0);
	if (n == 2){
	    SimpleNode target = (SimpleNode)node.jjtGetChild(1);
	    String s = (String)target.accept(this, context);
	    int id = target.id;
	    if (id == JJTIDNODE ||
		id == JJTINDEXNODE ||
		id == JJTSTATICMETHODNODE ||
		id == JJTSTATICMEMBERNODE ||
		id == JJTAPPLICATIONNODE ||
		id == JJTMETHODNODE ||
		id == JJTMEMBERNODE ||
		id == JJTCASTEXPRESSION ||
		id == JJTINTEGERNODE ||
		id == JJTFLOATINGNODE ||
		id == JJTCHARACTERNODE ||
		id == JJTSTRINGNODE ||
		id == JJTTRUENODE ||
		id == JJTFALSENODE ||
		id == JJTNULLNODE){
		return "(" + type + ")" + s;
	    } else {
		return "(" + type + ")(" + s + ")";
	    }
	} else {
	    return type;
	}
    }

    public Object classNode(SimpleNode node, Context context){
	int n = node.jjtGetNumChildren();
	String s = "class " + node.str;
	int dim = 0;
	for (int i = 0; i < n; i++){
	    SimpleNode ch = (SimpleNode)node.jjtGetChild(i);
	    if (ch.id == JJTPACKAGE){
		s += ".";
		s += ch.str;
	    } else if (ch.id == JJTARRAYTYPE){
		dim++;
	    }
	}
	for (int i = 0; i < dim; i++){
	    s += "[]";
	}
	return s;
    }

    public Object rangeNode(SimpleNode node, Context context){
	if (node.jjtGetNumChildren() == 3){
	    return accept(node, 0) + "[" + accept(node, 1) + ".." + accept(node, 2) + "]";
	} else {
	    return accept(node, 0) + "[" + accept(node, 1) + "..]";
	}
    }

    public Object indexNode(SimpleNode node, Context context){
	return accept(node, 0) + "[" + accept(node, 1) + "]";
    }

    public Object methodNode(SimpleNode node, Context context){
	String target = accept(node, 0);
	int id = ((SimpleNode)node.jjtGetChild(0)).id;
	if (id == JJTINTEGERNODE || id == JJTFLOATINGNODE){
	    target = "(" + target + ")";
	}
	String list = accept(node, 1);
	return target + "." + node.str + "(" + list.substring(1, list.length() - 1) + ")";
    }

    public Object staticMethodNode(SimpleNode node, Context context){
	String s = accept(node, 0) + "::" + node.str;
	String list = accept(node, 1);
	return s + "(" + list.substring(1, list.length() - 1) + ")";
    }

    public Object memberNode(SimpleNode node, Context context){
	String target = accept(node, 0);
	int id = ((SimpleNode)node.jjtGetChild(0)).id;
	if (id == JJTINTEGERNODE || id == JJTFLOATINGNODE){
	    target = "(" + target + ")";
	}
	return target + "." + node.str;
    }

    public Object staticMemberNode(SimpleNode node, Context context){
	return accept(node, 0) + "::" + node.str;
    }

    public Object applicationNode(SimpleNode node, Context context){
	int n = node.jjtGetNumChildren();
	String s = accept(node, 0) + "(";
	String list = accept(node, 1);
	return s + list.substring(1, list.length() - 1) + ")";
    }

    public Object integerNode(SimpleNode node, Context context){
	return node.str;
    }

    public Object floatingNode(SimpleNode node, Context context){
	return node.str;
    }

    public Object characterNode(SimpleNode node, Context context){
	return node.str;
    }

    public Object stringNode(SimpleNode node, Context context){
	if (escape){
	    StringBuffer b = new StringBuffer();
	    char c[] = node.str.toCharArray();
	    for (int i = 0; i < c.length; i++){
		if (c[i] == '"' || c[i] == '\\'){
		    b.append('\\');
		}
		b.append(c[i]);
	    }
	    return b.toString();
	} else {
	    return node.str;
	}
    }

    public Object trueNode(SimpleNode node, Context context){
	return "true";
    }

    public Object falseNode(SimpleNode node, Context context){
	return "false";
    }

    public Object nullNode(SimpleNode node, Context context){
	return "null";
    }

    public Object assignment(SimpleNode node, Context context){
	return binary(node, "=");
    }

    public Object assignmentTA(SimpleNode node, Context context){
	return binary(node, "*=");
    }

    public Object assignmentMA(SimpleNode node, Context context){
	return binary(node, "%=");
    }

    public Object assignmentDA(SimpleNode node, Context context){
	return binary(node, "/=");
    }

    public Object assignmentPA(SimpleNode node, Context context){
	return binary(node, "+=");
    }

    public Object assignmentSA(SimpleNode node, Context context){
	return binary(node, "-=");
    }

    public Object assignmentLA(SimpleNode node, Context context){
	return binary(node, "<<=");
    }

    public Object assignmentRA(SimpleNode node, Context context){
	return binary(node, ">>=");
    }

    public Object assignmentRAA(SimpleNode node, Context context){
	return binary(node, ">>>=");
    }

    public Object assignmentAA(SimpleNode node, Context context){
	return binary(node, "&=");
    }

    public Object assignmentEA(SimpleNode node, Context context){
	return binary(node, "^=");
    }

    public Object assignmentOA(SimpleNode node, Context context){
	return binary(node, "|=");
    }

    public Object orNode(SimpleNode node, Context context){
	return binary(node, "||");
    }

    public Object andNode(SimpleNode node, Context context){
	return binary(node, "&&");
    }

    public Object logOrNode(SimpleNode node, Context context){
	return binary(node, "|");
    }

    public Object logExorNode(SimpleNode node, Context context){
	return binary(node, "^");
    }

    public Object logAndNode(SimpleNode node, Context context){
	return binary(node, "&");
    }

    public Object equalNode(SimpleNode node, Context context){
	return binary(node, "==");
    }

    public Object notEqNode(SimpleNode node, Context context){
	return binary(node, "!=");
    }

    public Object instanceofExpression(SimpleNode node, Context context){
	return binary(node, "instanceof");
    }

    public Object ltNode(SimpleNode node, Context context){
	return binary(node, "<");
    }

    public Object gtNode(SimpleNode node, Context context){
	return binary(node, ">");
    }

    public Object leNode(SimpleNode node, Context context){
	return binary(node, "<=");
    }

    public Object geNode(SimpleNode node, Context context){
	return binary(node, ">=");
    }

    public Object shiftLeftNode(SimpleNode node, Context context){
	return binary(node, "<<");
    }

    public Object shiftRightNode(SimpleNode node, Context context){
	return binary(node, ">>");
    }

    public Object shiftArithmeticNode(SimpleNode node, Context context){
	return binary(node, ">>>");
    }

    public Object addNode(SimpleNode node, Context context){
	return binary(node, "+");
    }

    public Object subtractNode(SimpleNode node, Context context){
	return binary(node, "-");
    }

    public Object multNode(SimpleNode node, Context context){
	return binary(node, "*");
    }

    public Object divideNode(SimpleNode node, Context context){
 	return binary(node, "/");
    }

    public Object modNode(SimpleNode node, Context context){
 	return binary(node, "%");
    }

    public Object negativeNode(SimpleNode node, Context context){
	return "-" + operand(node, 0);
    }

    public Object preIncrNode(SimpleNode node, Context context){
	return "++" + operand(node, 0);
    }

    public Object preDecrNode(SimpleNode node, Context context){
	return "--" + operand(node, 0);
    }

    public Object postIncrNode(SimpleNode node, Context context){
	return operand(node, 0) + "++";
    }

    public Object postDecrNode(SimpleNode node, Context context){
	return operand(node, 0) + "--";
    }

    public Object logNotNode(SimpleNode node, Context context){
	return "~" + operand(node, 0);
    }

    public Object notNode(SimpleNode node, Context context){
	return "!" + operand(node, 0);
    }

    public Object continueNode(SimpleNode node, Context context){
	return "continue";
    }

    public Object breakNode(SimpleNode node, Context context){
	return "break";
    }
    public Object returnNode(SimpleNode node, Context context){
	int n = node.jjtGetNumChildren();
	String s = "return";
	if (n == 1){
	    s += " " + accept(node, 0);
	}
	return s;
    }

    private static String indent = " ";
    private int nest = 0;

    public Object blockNode(SimpleNode node, Context context){
	int n = node.jjtGetNumChildren();
	if (n > 1){
	    String s = null;
	    if (escape){
		s = "{\\n";
	    } else {
		s = "{\n";
	    }
	    ++nest;
	    for (int i = 0; i < nest; i++){
		s += indent;
	    }
	    s += accept(node, 0);
	    for (int i = 1; i < n; i++){
		if (escape){
		    s += "\\n";
		} else {
		    s += "\n";
		}
		for (int j = 0; j < nest; j++){
		    s += indent;
		}
		s += accept(node, i);
	    }
	    --nest;
	    if (escape){
		s += "\\n";
	    } else {
		s += "\n";
	    }
	    for (int i = 0; i < nest; i++){
		s += indent;
	    }
	    return s + "}";
	} else {
	    return "{}";
	}
    }

    public Object ifStatement(SimpleNode node, Context context){
	int n = node.jjtGetNumChildren();
	String s = "if (" + accept(node, 0) + ") " + accept(node, 1);
	for (int i = 2; i < n; i++){
	    SimpleNode c = (SimpleNode)node.jjtGetChild(i);
	    if (c.id == JJTELSEIFNODE){
 		s += " else if (" + accept(c, 0) + ") " + accept(c, 1);
	    } else if (c.id == JJTELSENODE){
 		s += " else " + accept(c, 0);
	    }
	}
	return s;
    }

    public Object whileStatement(SimpleNode node, Context context){
	return "while (" + accept(node, 0) + ") " + accept(node, 1);
    }

    public Object forStatement(SimpleNode node, Context context){
	SimpleNode initNode = null;
	SimpleNode condNode = null;
	SimpleNode updateNode = null;
	SimpleNode blockNode = null;

	int j = 0;
	SimpleNode n = (SimpleNode)node.jjtGetChild(j);
	if (n.id == JJTFORINIT){
	    initNode = n;
	    j++;
	}
	n = (SimpleNode)node.jjtGetChild(j);
	if (n.id != JJTFORUPDATE && n.id != JJTBLOCK){
	    condNode = n;
	    j++;
	}
	n = (SimpleNode)node.jjtGetChild(j);
	if (n.id == JJTFORUPDATE){
	    updateNode = n;
	    j++;
	}
	blockNode = (SimpleNode)node.jjtGetChild(j);

	StringBuffer buf = new StringBuffer("for (");
	if (initNode != null){
	    int num = initNode.jjtGetNumChildren();
	    SimpleNode sn = null;
	    if (num > 0){
		sn = (SimpleNode)initNode.jjtGetChild(0);
		buf.append(sn.str + "=" + accept(sn, 0));
	    }
	    for (int i = 1; i < num; i++){
		sn = (SimpleNode)initNode.jjtGetChild(i);
		buf.append("," + sn.str + "=" + accept(sn, 0));
	    }
	}
	buf.append(';');
	if (condNode != null){
	    buf.append(condNode.accept(this, context));
	}
	buf.append(';');
	if (updateNode != null){
	    buf.append(expressionList(updateNode, context, ','));
	}
	buf.append(") " + blockNode.accept(this, context));
	return buf.toString();
    }

    public Object foreachStatement(SimpleNode node, Context context){
	return "foreach " + node.str + " (" + accept(node, 0) + ") " + accept(node, 1);
    }

    public Object switchStatement(SimpleNode node, Context context){
	String s =  "switch (" + accept(node, 0) + "){";
	for (int i = 1; i < node.jjtGetNumChildren(); i++){
	    SimpleNode n = (SimpleNode)node.jjtGetChild(i);
	    if (n.jjtGetNumChildren() == 1){ // case
		s += "case " + accept(n, 0) + ": " + accept(node, ++i);
	    } else {
		s += "default: " + accept(node, ++i);
	    }
	    s += "; ";
	}
	return s + "}";
    }

    public Object switchBlock(SimpleNode node, Context context){
	int n = node.jjtGetNumChildren();
	String s = accept(node, 0);
	for (int i = 1; i < n; i++){
	    s += "; " + accept(node, i);
	}
	return s;
    }

    public Object functionStatement(SimpleNode node, Context context){
	String s = "(function ";
	if (node.str != null){
	    s += node.str;
	}
	SimpleNode args = (SimpleNode)node.jjtGetChild(0);
	int nargs = args.jjtGetNumChildren();
	s += "(";
	if (nargs > 0){
	    s += ((SimpleNode)args.jjtGetChild(0)).str;
	    for (int i = 1; i < nargs; i++){
		s += ", " + ((SimpleNode)args.jjtGetChild(i)).str;
	    }
	}
	return s +  ") " + accept(node, 1) + ")";
    }

    final String accept(SimpleNode node, int idx){
	return (String)node.jjtGetChild(idx).accept(this, null);
    }
}

