/*
 * @(#) JarResourceConnection.java	1.1 98/08/06
 *
 * Copyright (c) 1997,1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.ext.jar;

import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;
import java.io.InputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.BufferedInputStream;
import java.lang.reflect.Method;
import pnuts.ext.JarClassLoader;

/**
 * URLConnection for resource in a JAR file
 * The URL scheme is same as jdk1.2's.
 *
 * @version	1.1
 * @author	Toyokazu Tomatsu
 */
class JarResourceConnection extends URLConnection {
    private JarClassLoader loader;
    private String name;	// name of the resource

    protected JarResourceConnection (URL url)
		throws MalformedURLException, IOException
    {
	super(url);
	String file = url.getFile();
	if (file.charAt(0) == '/'){
	    file = file.substring(1);
	}
	int idx = file.indexOf("!/");
	if (idx < 0){
	    throw new MalformedURLException();
	}
	String sub = file.substring(0, idx);
	name = file.substring(idx + 2);
	loader = JarClassLoader.getLoader(sub);
	if (loader == null){
	    URL subUrl = new URL(sub);
	    loader = new JarClassLoader();
	    loader.loadJar(subUrl);
	}
	if (!setup){
	    setup = true;
	    setupMimeTypes(loader);
	}
    }

    public void connect() throws IOException {
    }

    public String getContentType() {
	String type = null;
	if (mimeTypes != null){
	    try {
		/*
		 * call MimetypesFileTypeMap.getContentType()
		 */
		type = (String)mimeMethod.invoke(mimeTypes, new Object[]{name});
		if (type != null){
		    return type;
		}
	    } catch (Exception e){
		/* ignore */
	    }
	}
	try {
	    type = guessContentTypeFromStream(new BufferedInputStream(getInputStream()));
	} catch (IOException e){/* ignore */}
	if (type == null){
	    type = guessContentTypeFromName(name);
	}
	if (type != null){
	    return type;
	} else {
	    return "content/unknown";
	}
    }

    public InputStream getInputStream() throws IOException {
	InputStream in = loader.getResourceAsStream(name);
	if (in == null){
	    throw new FileNotFoundException(url.toString());
	}
	return in;
    }

    static Object mimeTypes = null;
    static Method mimeMethod = null;
    static boolean setup = false;
    static String jafMimeMapClassName = "javax.activation.MimetypesFileTypeMap";

    static void setupMimeTypes(ClassLoader loader){
	Class clazz = null;
	try {
	    clazz = loader.loadClass(jafMimeMapClassName);
	} catch (ClassNotFoundException e){
	    /* ignore */
	}
	if (clazz != null){
	    try {
		mimeTypes = clazz.newInstance();
		mimeMethod = clazz.getMethod("getContentType", new Class[]{String.class});
	    } catch (Exception e2){
		/* ignore */
	    }
	}
    }
}
