/*
 * @(#) ClassHolder.java	1.2 99/03/31
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

import java.io.Serializable;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.util.Hashtable;

/**
 * This class is used in a Serializable class which has Class objects in its members;
 *  <pre>
 *   Usage: 
 *    (1) in writeObject() method:
 *             if (member instanceof Class){
 *                  stream.writeObject(new ClassHolder((Class)member));
 *             } else {
 *                  stream.writeObject(member);
 *             }
 *
 *    (2) in readObject() method:
 *             Object o = readObject();
 *             if (o instanceof ClassHolder){
 *                   o = ((ClassHolder)o).get();
 *             }
 *   </pre>
 *
 * @version	1.2
 * @author	Toyokazu Tomatsu
 */
class ClassHolder implements Serializable {

    static final long serialVersionUID = -2110913058378048904L;

    static Hashtable primType = new Hashtable();
    static {
	String[] prim = {"byte", "short", "int", "long",
			 "float", "double", "char", "boolean", "void"};
	Class[] type = {Byte.TYPE, Short.TYPE, Integer.TYPE, Long.TYPE,
			Float.TYPE, Double.TYPE, Character.TYPE, Boolean.TYPE, Void.TYPE};
	for (int i = 0; i < prim.length; i++){
	    primType.put(prim[i], type[i]);
	}
    }

    transient Class clazz;

    ClassHolder(Class clazz){
	this.clazz = clazz;
    }

    public Class get(){
	return clazz;
    }

    private synchronized void writeObject(ObjectOutputStream s) throws IOException {
	s.defaultWriteObject();
	if (!Pnuts.isJava2() && !(Serializable.class.isAssignableFrom(clazz)) || clazz.isPrimitive()){
	    s.writeObject(clazz.getName());
	} else {
	    s.writeObject(clazz);
	}
    }

    private synchronized void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException {
	s.defaultReadObject();
	Object o = s.readObject();
	if (o instanceof Class){
	    clazz = (Class)o;
	} else if (o instanceof String){
	    clazz = (Class)primType.get(o);
	    if (clazz == null){
		clazz = Pnuts.loadClass((String)o);
	    }
	}
    }
}
