/*
 * @(#) Format.java	1.3 99/03/26
 *
 * Copyright (c) 1997,1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

import java.lang.reflect.Array;
    
class Format {

    static char[] buf = new char[1];

    private static char[] hexDigit = {
	'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'
    };

    private static char toHex(int nibble) {
	return hexDigit[(nibble & 0xF)];
    }

    static String listToString(Object o){
	if (o == null){
	    return "null";
	} else if (!(o.getClass().isArray())){
	    if (o instanceof String){
		return "\"" + o.toString() + "\"";
	    } else if (o instanceof Character){
		char ch = ((Character)o).charValue();
		switch (ch) {
		  case '\\': return "'\\'";
		  case '\b': return "'\\b'";
		  case '\f': return "'\\f'";
		  case '\t': return "'\\t'";
		  case '\n': return "'\\n'";
		  case '\r': return "'\\r'";
		  case '\'': return "'\\''";
		  case '?': return "'?'";
		  default:
		      buf[0] = ch;
		      byte[] conv = new String(buf).getBytes();
		      if (!Character.isISOControl(ch) &&
			  !(conv.length == 1 && conv[0] == (byte)'?')){
			  return "'" + ch + "'";
		      }
		      return "'\\u" +
			  toHex((ch >> 12) & 0xF) + 
			  toHex((ch >>  8) & 0xF) +
			  toHex((ch >>  4) & 0xF) + 
			  toHex((ch >>  0) & 0xF) + "'";
		}
	    } else if (o instanceof Class){
		Class c = (Class)o;
		return getClassName(c) + getSuffix(c);
	    } else if (o instanceof Byte){
		return "#" + Integer.toString((((Byte)o).byteValue() + 256) % 256, 16);
	    } else {
		return o.toString();
	    }
	}
	String s = "";
	int length = Array.getLength(o);
	s += "[";
	if (length > 0){
	    s += listToString(Array.get(o, 0));
	}
	for (int j = 1; j < length; j++){
	    s += ", ";
	    s += listToString(Array.get(o, j));
	}
	s += "]";
	return s;
    }

    static String getClassName(Class c){
	if (c.isArray()){
	    StringBuffer buf = new StringBuffer();
	    for (; c.isArray(); c = c.getComponentType()){
		buf.append("[]");
	    }
	    buf.insert(0, c.getName());
	    return buf.toString();
	} else {
	    return c.getName();
	}
    }

    static String getSuffix(Class c){
	if (c.isInterface()){
	    return " interface";
	} else if (c.isPrimitive()){
	    return " type";
	} else {
	    return " class";
	}
    }
}
