/*
 * @(#) ImportEnv.java	1.12 99/06/02
 *
 * Copyright (c) 1997,1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

class ImportEnv implements Serializable, Cloneable {

    static final long serialVersionUID = 2987083055138815991L;

    /**
     * @serial
     */
    Hashtable imports = new Hashtable(16);

    /**
     * @serial
     */
    Hashtable importedClass = new Hashtable(16);

    /**
     * @serial
     */
    Vector pkgOrder = new Vector();

    /**
     * @serial
     */
    Vector classOrder = new Vector();

    transient Hashtable cache = new Hashtable(32);
    transient Hashtable failCache = new Hashtable(32);

    Import getImport(String name){
	Import imp = (Import)imports.get(name);
	if (imp != null){
	    return imp;
	} else {
	    return new Import(name);
	}
    }

    void add(Class clazz){
	String name = clazz.getName();
	int idx = name.lastIndexOf(".");
	String classname = name.substring(idx + 1);
	ClassHolder c = (ClassHolder)importedClass.get(classname);
	if (c != null){
	    classOrder.removeElement(c);
	} else {
	    c = new ClassHolder(clazz);
	}
	classOrder.addElement(c);
	importedClass.put(classname, c);
	cache.put(classname, clazz);
    }

    public void add(String pkgname){
	Import imp = (Import)imports.get(pkgname);
	if (imp != null){
	    pkgOrder.removeElement(imp);
	} else {
	    imp = getImport(pkgname);
	}
	pkgOrder.addElement(imp);
	imports.put(pkgname, imp);
	cache = new Hashtable(32);
	failCache = new Hashtable(32);
    }
 
    public Class get(String sym){
	Object v = cache.get(sym);
	if (v != null){
	    return (Class)v;
	}
	v = importedClass.get(sym);
	if (v != null){
	    return ((ClassHolder)v).get();
	}
	if (Package.globalPackage.autoloadTable.get(sym) != null){
	    return null;
	}
	int symlen = sym.length();
	StringBuffer buf = new StringBuffer(sym);
	boolean lowercase = !Character.isUpperCase(sym.charAt(0));

 	int size = pkgOrder.size();
	for (int i = size - 1; i >= 0; i--){
	    Import imp = (Import)pkgOrder.elementAt(i);
	    buf.setLength(symlen);
	    String name = imp.getName();
	    if (lowercase && name.startsWith("java.")){
		continue;
	    }
	    buf.append(name);
	    String key = buf.toString();
	    if (failCache.get(key) != null){
		continue;
	    }
	    Class value = imp.get(sym);
	    if (value != null){
		cache.put(sym, value);
		return value;
	    } else {
		failCache.put(key, key);
	    }
	}
	return null;
    }

    // class java.awt.hogehoge
    // put("java.awt", "hogehoge")

    public void put(String pkg, String sym){
	Import imp = getImport(pkg);
	imp.get(sym);
    }

    Object[] list(){
	Object ret[] = new Object[pkgOrder.size() + classOrder.size()];
	int i = ret.length - 1;
	for (Enumeration e = pkgOrder.elements(); e.hasMoreElements(); ){
	    String name = ((Import)e.nextElement()).name;
	    if ("".equals(name)){
		ret[i--] = "*";
	    } else {
		ret[i--] = name + ".*";
	    }
	}
	for (Enumeration e = classOrder.elements(); e.hasMoreElements(); ){
	    ret[i--] = ((ClassHolder)e.nextElement()).get();
	}
	return ret;
    }

    public Object clone(){
	try {
	    ImportEnv c = (ImportEnv)super.clone();
	    c.imports = (Hashtable)imports.clone();
	    c.importedClass = (Hashtable)importedClass.clone();
	    c.pkgOrder = (Vector)pkgOrder.clone();
	    c.classOrder = (Vector)classOrder.clone();
	    c.cache = new Hashtable(32);
	    return c;
	} catch (Throwable t){
	    throw new InternalError();
	}
    }

    private synchronized void readObject(ObjectInputStream s) throws IOException, ClassNotFoundException {
	s.defaultReadObject();
	cache = new Hashtable(32);
	failCache = new Hashtable(32);
    }
}
