/*
 * @(#) Numeric.java	1.4 99/02/08
 *
 * Copyright (c) 1997,1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

/**
 * In Pnuts, arithmetic operations for objects implements this interface
 * causes a call of the corresponding methods in this interface.
 * See <a href="/pnuts/doc/lang.html#sugar">Pnuts Language Specification</a>
 * for details.
 *
 * @version	1.4
 * @author 	Toyokazu Tomatsu
 */
public interface Numeric {
    /**
     * adds the value of parameter to itself
     */
    Object add(Object obj);

    /**
     * subtracts the value of parameter from the object
     */
    Object subtract(Object obj);

    /**
     * multiplies itself with the value of parameter
     */
    Object multiply(Object o);

    /**
     * divides itself by the value of parameter
     */
    Object divide(Object obj);

    /**
     * negates itself by the value of parameter
     */
    Object negate();

    /**
     * inverts itself
     */
    Object inverse();

    /**
     * compare the object with the parameter.
     * returns one of the followings:
     * NOT_EQUAL, LEFT_IS_BIGGER, RIGHT_IS_BIGGER, EQUAL
     */
    int compareTo(Object o); // >, <, <=, >=, ==, !=

    int NOT_EQUAL = 2;
    int LEFT_IS_BIGGER = 1;
    int RIGHT_IS_BIGGER = -1;
    int EQUAL = 0;
}
