/*
 * @(#) PnutsException.java	1.12 99/03/30
 *
 * Copyright (c) 1997,1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.ResourceBundle;
import java.text.MessageFormat;
import java.io.PrintWriter;

/**
 * This is a wrapper class for Exception to be thrown.
 *
 * @version	1.12
 * @author	Toyokazu Tomatsu
 */
public class PnutsException extends RuntimeException {

    final static Object[] NO_PARAM = new Object[]{};

    /**
     * @serial
     */
    protected int line;

    /**
     * @serial
     */
    protected String file;

    /**
     * @serial
     */
    protected Throwable throwable;

    /**
     * @serial
     */
    protected Vector trace;

    /**
     * @serial
     */
    protected String contextName;

    public PnutsException(){
	this("");
    }

    public PnutsException(String msg){
	super(msg);
	throwable = this;
    }

    public PnutsException(String msg, Context context){
	this(msg);

	while (context.eval && context.parent != null){
	    context = context.parent;
	}
	this.file = context.getFile();
	this.line = context.beginLine;
	this.contextName = context.name;
    }

    /**
     * this constructor creates a PnutsException using i18n resources in pnuts.properties.
     */
    public PnutsException(String key, Object param[], Context context){
	this(getMessage(key, param), context);
    }

    /**
     * this constructor creates a PnutsException using i18n resources in pnuts.properties.
     */
    public PnutsException(Throwable t){
	this.throwable = t;
    }

    public PnutsException(Throwable t, Context context){

	while (context.eval && context.parent != null){
	    context = context.parent;
	}

	if (t instanceof PnutsException){
	    PnutsException p = (PnutsException)t;
	    this.throwable = p.throwable;
	    this.trace = p.trace;
	    this.file = p.file;
	    this.line = p.line;
	} else {
	    this.throwable = t;
	    this.line = context.beginLine;
	    this.file = context.getFile();
	}
	this.contextName = context.name;
    }

    public Throwable getThrowable(){
	return throwable;
    }

    static String getMessage(String key, Object a[]){
	ResourceBundle bundle = ResourceBundle.getBundle("pnuts");
	if (bundle == null){
	    return null;
	}
	String fmt = bundle.getString(key);
	if (fmt == null){
	    return null;
	}
	return MessageFormat.format(fmt, a);
    }

    public void printStackTrace(){
	printStackTrace(System.err);
    }

    String position(){
	StringBuffer buf = new StringBuffer();
	if (file != null || contextName != null){
	    buf.append('[');
	    if (contextName != null){
		buf.append(contextName + ", ");
	    }
	    if (file != null){
		buf.append("file: " + file);
		if (line > 0){
		    buf.append(" line: " + line);
		}
	    } else if (line > 0){
		buf.append("line: " + line);
	    }
	    buf.append(']');
	}
	if (buf.length() > 0){
	    buf.append(": ");
	}
	return buf.toString();
    }

    public void printStackTrace(PrintWriter writer){
//	writer.print(position());
	if (throwable != null && throwable != this){
	    throwable.printStackTrace(writer);
	} else {
	    super.printStackTrace(writer);
	}
    }

    public String toString(){
	String msg = throwable.getMessage();
	String name = throwable.getClass().getName() + " ";
	String s = position();
 	s += name + ": " + msg;
	if (trace != null && trace.size() > 0){
	    int i = 0;
	    for (Enumeration e = trace.elements(); e.hasMoreElements(); ){
		if (++i > 10){
		    s += "\n\t...";
		    break;
		}
		s += "\n";
		s += "\t";
		s += e.nextElement();
	    }
	}
	return s;
    }
}
