/*
 * %Z% %M%	%I% %E%
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.lang;

import java.io.Serializable;
import java.io.IOException;
import java.io.ObjectInputStream;

public class SimpleNode implements /*Node,*/ PnutsParserTreeConstants, Serializable/*, Cloneable*/ {

    static final long serialVersionUID = 4797980214789556513L;

    /**
     * @serial
     */
    protected SimpleNode parent;

    /**
     * @serial
     */
    protected SimpleNode[] children;

    /**
     * @serial
     */
    public int id;

    SimpleNode(int i) {
	id = i;
    }

/*
    public void jjtOpen() {
    }

    public void jjtClose() {
    }
*/
  
    public void jjtSetParent(SimpleNode n) { parent = n; }
    public SimpleNode jjtGetParent() { return parent; }

    public void jjtAddChild(SimpleNode n, int i) {
	if (children == null) {
	    children = new SimpleNode[i + 1];
	} else if (i >= children.length) {
	    SimpleNode c[] = new SimpleNode[i + 1];
	    System.arraycopy(children, 0, c, 0, children.length);
	    children = c;
	}
	children[i] = n;
    }

    public final SimpleNode jjtGetChild(int i) {
	return children[i];
    }

    public final int jjtGetNumChildren() {
	return (children == null) ? 0 : children.length;
    }

/**/
    public String toString() { return PnutsParserTreeConstants.jjtNodeName[id]; }
    public String toString(String prefix) { return prefix + PnutsParserTreeConstants.jjtNodeName[id]; }

    public void dump(String prefix) {
	System.out.println(toString(prefix));
	if (children != null) {
	    for (int i = 0; i < children.length; ++i) {
		SimpleNode n = (SimpleNode)children[i];
		if (n != null) {
		    n.dump(prefix + " ");
		}
	    }
	}
    }
/**/

    // Pnuts specific attributes

    /**
     * @serial
     */
    public String str;

    /**
     * @serial
     */
    public int beginLine;

    /**
     * Value of the node. Act as cache.
     */
    transient Object value;

    /**
     * Used differently by visitors.
     */
    public transient Object info;

    /**
     * dispatch the instance to a Visitor
     */
    public Object accept(Visitor visitor, Context context) {
	switch (id) {
	case JJTSTART:
	    return visitor.start(this, context);
	case JJTSTARTSET:
	    return visitor.startSet(this, context);
	case JJTEXPRESSIONLIST:
	    return visitor.expressionList(this, context);
	case JJTIDNODE:
	    return visitor.idNode(this, context);
	case JJTGLOBAL:
	    return visitor.global(this, context);
	case JJTARRAYTYPE:
	    return visitor.arrayType(this, context);
	case JJTLISTELEMENTS:
	    return visitor.listElements(this, context);
	case JJTCASTEXPRESSION:
	    return visitor.castExpression(this, context);
	case JJTCLASS:
	    return visitor.classNode(this, context);
	case JJTINDEXNODE:
	    return visitor.indexNode(this, context);
	case JJTRANGENODE:
	    return visitor.rangeNode(this, context);
	case JJTMETHODNODE:
	    return visitor.methodNode(this, context);
	case JJTSTATICMETHODNODE:
	    return visitor.staticMethodNode(this, context);
	case JJTMEMBERNODE:
	    return visitor.memberNode(this, context);
	case JJTSTATICMEMBERNODE:
	    return visitor.staticMemberNode(this, context);
	case JJTAPPLICATIONNODE:
	    return visitor.applicationNode(this, context);
	case JJTINTEGERNODE:
	    return visitor.integerNode(this, context);
	case JJTFLOATINGNODE:
	    return visitor.floatingNode(this, context);
	case JJTCHARACTERNODE:
	    return visitor.characterNode(this, context);
	case JJTSTRINGNODE:
	    return visitor.stringNode(this, context);
	case JJTTRUENODE:
	    return visitor.trueNode(this, context);
	case JJTFALSENODE:
	    return visitor.falseNode(this, context);
	case JJTNULLNODE:
	    return visitor.nullNode(this, context);
	case JJTASSIGNMENT:
	    return visitor.assignment(this, context);
	case JJTASSIGNMENTTA:
	    return visitor.assignmentTA(this, context);
	case JJTASSIGNMENTMA:
	    return visitor.assignmentMA(this, context);
	case JJTASSIGNMENTDA:
	    return visitor.assignmentDA(this, context);
	case JJTASSIGNMENTPA:
	    return visitor.assignmentPA(this, context);
	case JJTASSIGNMENTSA:
	    return visitor.assignmentSA(this, context);
	case JJTASSIGNMENTLA:
	    return visitor.assignmentLA(this, context);
	case JJTASSIGNMENTRA:
	    return visitor.assignmentRA(this, context);
	case JJTASSIGNMENTRAA:
	    return visitor.assignmentRAA(this, context);
	case JJTASSIGNMENTAA:
	    return visitor.assignmentAA(this, context);
	case JJTASSIGNMENTEA:
	    return visitor.assignmentEA(this, context);
	case JJTASSIGNMENTOA:
	    return visitor.assignmentOA(this, context);
	case JJTORNODE:
	    return visitor.orNode(this, context);
	case JJTANDNODE:
	    return visitor.andNode(this, context);
	case JJTLOGORNODE:
	    return visitor.logOrNode(this, context);
	case JJTLOGEXORNODE:
	    return visitor.logExorNode(this, context);
	case JJTLOGANDNODE:
	    return visitor.logAndNode(this, context);
	case JJTEQUALNODE:
	    return visitor.equalNode(this, context);
	case JJTNOTEQNODE:
	    return visitor.notEqNode(this, context);
	case JJTINSTANCEOFEXPRESSION:
	    return visitor.instanceofExpression(this, context);
	case JJTLTNODE:
	    return visitor.ltNode(this, context);
	case JJTGTNODE:
	    return visitor.gtNode(this, context);
	case JJTLENODE:
	    return visitor.leNode(this, context);
	case JJTGENODE:
	    return visitor.geNode(this, context);
	case JJTSHIFTLEFTNODE:
	    return visitor.shiftLeftNode(this, context);
	case JJTSHIFTRIGHTNODE:
	    return visitor.shiftRightNode(this, context);
	case JJTSHIFTARITHMETICNODE:
	    return visitor.shiftArithmeticNode(this, context);
	case JJTADDNODE:
	    return visitor.addNode(this, context);
	case JJTSUBTRACTNODE:
	    return visitor.subtractNode(this, context);
	case JJTMULTNODE:
	    return visitor.multNode(this, context);
	case JJTDIVIDENODE:
	    return visitor.divideNode(this, context);
	case JJTMODNODE:
	    return visitor.modNode(this, context);
	case JJTNEGATIVENODE:
	    return visitor.negativeNode(this, context);
	case JJTPREINCRNODE:
	    return visitor.preIncrNode(this, context);
	case JJTPREDECRNODE:
	    return visitor.preDecrNode(this, context);
	case JJTLOGNOTNODE:
	    return visitor.logNotNode(this, context);
	case JJTNOTNODE:
	    return visitor.notNode(this, context);
	case JJTPOSTINCRNODE:
	    return visitor.postIncrNode(this, context);
	case JJTPOSTDECRNODE:
	    return visitor.postDecrNode(this, context);
	case JJTBREAK:
	    return visitor.breakNode(this, context);
	case JJTCONTINUE:
	    return visitor.continueNode(this, context);
	case JJTRETURN:
	    return visitor.returnNode(this, context);
	case JJTBLOCK:
	    return visitor.blockNode(this, context);
	case JJTIFSTATEMENT:
	    return visitor.ifStatement(this, context);
	case JJTWHILESTATEMENT:
	    return visitor.whileStatement(this, context);
	case JJTFORSTATEMENT:
	    return visitor.forStatement(this, context);
	case JJTFOREACHSTATEMENT:
	    return visitor.foreachStatement(this, context);
	case JJTSWITCHSTATEMENT:
	    return visitor.switchStatement(this, context);
	case JJTSWITCHBLOCK:
	    return visitor.switchBlock(this, context);
	case JJTFUNCTIONSTATEMENT:
	    return visitor.functionStatement(this, context);
	case JJTPACKAGE:
	case JJTELSEIFNODE:
	case JJTELSENODE:
	case JJTSWITCHLABEL:
	case JJTPARAMLIST:
	case JJTPARAM:
	default:
	    return null;
	}
    }

    private synchronized void readObject(ObjectInputStream s)
         throws IOException, ClassNotFoundException
    {
	s.defaultReadObject();
	switch (id){
	case JJTIDNODE:
	case JJTGLOBAL:
	case JJTMETHODNODE:
	case JJTMEMBERNODE:
	case JJTSTATICMETHODNODE:
	case JJTSTATICMEMBERNODE:
	case JJTLOCAL:
	case JJTPARAM:
	    str = str.intern();
	    break;
	case JJTFUNCTIONSTATEMENT:
	    if (str != null){
		str = str.intern();
	    }
	case JJTSTART:
	    info = new int[]{1};
	    break;
	case JJTFOREACHSTATEMENT:
	    str = str.intern();
	case JJTBLOCK:
	case JJTSWITCHBLOCK:
	case JJTWHILESTATEMENT:
	case JJTFORSTATEMENT:
	case JJTSWITCHSTATEMENT:
	    info = new Object[]{Boolean.FALSE, Boolean.FALSE, null};
	    break;
	}
    }

    /*
    protected Object clone() {
	try {
	    SimpleNode n = (SimpleNode)super.clone();
	    if (children != null){
		n.children = new SimpleNode[children.length];
		System.arraycopy(children, 0, n.children, 0, children.length);
	    }
	    return n;
	} catch (CloneNotSupportedException e){
	    throw new InternalError();
	}
    }
    */
}
