/*
 * @(#) CompileContext.java	1.4 99/06/03
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.trans;

import pnuts.lang.Package;
import pnuts.lang.Context;
import java.util.Hashtable;
import java.io.Writer;
import java.io.PrintWriter;
import java.io.CharArrayWriter;
import java.io.IOException;

/**
 * This class is used with pnuts.trans.Translator class to translate
 * Pnuts scripts to a Java class.
 *
 * @see pnuts.trans.Translator
 * @see pnuts.lang.Context
 * @version 	1.4
 */
public class CompileContext extends Context {

    Frame env = new Frame();
    Symbol sym = new Symbol();
    Hashtable constants = new Hashtable();
    String top = "top";
    CharArrayWriter cout = new CharArrayWriter();
    PrintWriter pw = new PrintWriter(cout);

    void emit(String s){
	pw.print(s);
    }

    public void writeTo(Writer out) throws IOException {
	cout.writeTo(out);
    }

    public Object clone(){
	try {
	    CompileContext c = (CompileContext)super.clone();
	    c.cout = new CharArrayWriter();
	    c.pw = new PrintWriter(c.cout);
	    return c;
	} catch (Throwable t){
	    throw new InternalError();
	}
    }

    void openFrame(String func, String locals[], String paramName){
	env = new Frame(locals, paramName, func, sym.gen(), env);
	_declare("!top");
	top = getReference("!top").symbol;
    }

    void closeFrame(){
	env = env.parent;
	Reference r = getReference("!top");
	if (r == null){
	    top = "top";
	} else {
	    top = r.symbol;
	}
    }

    void openScope(String locals[]){
	String s[] = new String[locals.length];
	for (int i = 0; i < s.length; i++){
	    s[i] = sym.gen();
	}
	env.openLocal(locals, s);
    }

    void closeScope(){
	env.closeLocal();
    }

    Reference getReference(String symbol){
	Frame f = env;
	String sym = symbol.intern();
	while (f != null){
	    Reference r = f.getReference(sym);
	    if (r != null){
		return r;
	    }
	    f = f.parent;
	}
	return null;
    }

    void _declare(String symbol){
	env._declare(symbol, sym.gen());
    }

    void _declare(String symbol, String assoc){
	env._declare(symbol, assoc);
    }

    void declare(String symbol){
	env.declare(symbol, sym.gen());
    }

    void declare(String symbol, String assoc){
	env.declare(symbol, assoc);
    }

    int getLine(){
	return beginLine;
    }

    void setLine(int line){
	beginLine = line;
    }
}
