/*
 * @(#) Frame.java	1.2 99/05/19
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.trans;

/**
 * This class represents a function scope.
 */
class Frame {
    Frame parent;
    SymbolSet symbolSet;
    SymbolSet bottom;
    String fname;
    String fsym;
    String[] locals;
    String paramName;

    Frame(){
	openLocal();
    }

    Frame(String locals[], String paramName, String fname, String fsym, Frame parent){
	this.parent = parent;
	if (fname != null){
	    this.fname = fname.intern();
	    this.fsym = fsym.intern();
	}
	for (int i = 0; i < locals.length; i++){
	    locals[i] = locals[i].intern();
	}
	this.locals = locals;
	this.paramName = paramName;
	this.bottom = new SymbolSet(null);
	this.symbolSet = bottom;
	openLocal();
    }

    void openLocal(){
	openLocal(new String[]{}, new String[]{});
    }

    void openLocal(String locals[], String keys[]){
	this.symbolSet = new SymbolSet(symbolSet);
	for (int i = 0; i < locals.length; i++){
	    symbolSet.add(locals[i], keys[i]);
	}
    }

    void closeLocal(){
	this.symbolSet = symbolSet.parent;
    }

    Reference getReference(String sym){
	String k = this.symbolSet.assoc(sym);
	if (k != null){
	    return new Reference(k, 0);
	}
	if (locals != null){
	    for (int i = 0; i < locals.length; i++){
		if (locals[i] == sym){
		    return new Reference(paramName, i);
		}
	    }
	}
	if (this.fname != null && this.fname == sym){
	    return new Reference(fsym, -1);
	}
	return null; // not found
    }

    public void declare(String symbol, String key){
	bottom.add(symbol, key);
    }

    public void _declare(String symbol, String key){
	symbolSet.add(symbol, key);
    }
}
