/*
 * @(#) SymbolSet.java	1.1 99/04/27
 *
 * Copyright (c) 1997-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.trans;

/**
 * This class represents a variable scope made by functions,
 * for/foreach statements, and so on.
 */
class SymbolSet {
    SymbolSet parent;
    String keys[] = new String[4];
    String maps[] = new String[4];
    int count = 0;

    SymbolSet(){
    }

    SymbolSet(SymbolSet parent){
	this.parent = parent;
    }

    void add(String sym, String map){
	if (count > keys.length - 1){
	    String newKeys[] = new String[keys.length * 2];
	    System.arraycopy(keys, 0, newKeys, 0, keys.length);
	    String newMaps[] = new String[maps.length * 2];
	    System.arraycopy(maps, 0, newMaps, 0, maps.length);
	    keys = newKeys;
	    maps = newMaps;
	}
	keys[count] = sym.intern();
	maps[count] = map.intern();
	count++;
    }

    String assoc(String key){
	return _assoc(key.intern());
    }

    String _assoc(String key){
	for (int i = 0; i < count; i++){
	    if (keys[i] == key){
		return maps[i];
	    }
	}
	if (parent != null){
	    return parent._assoc(key);
	} else {
	    return null;
	}
    }

    String getMap(String key){
	String k = key.intern();
	for (int i = 0; i < count; i++){
	    if (keys[i] == k){
		return maps[i];
	    }
	}
	return null;
    }

    String[] getMaps(String key){
	if (parent == null){
	    String m = getMap(key);
	    if (m == null){
		return new String[]{};
	    } else {
		return new String[]{m};
	    }
	} else {
	    String s1[] = parent.getMaps(key);
	    String s2 = getMap(key);
	    if (s2 != null){
		String s3[] = new String[s1.length + 1];
		System.arraycopy(s1, 0, s3, 0, s1.length);
		s3[s1.length] = s2;
		return s3;
	    } else {
		return s1;
	    }
	}
    }
}
