/*
 * @(#) DefaultClassLoader.java	1.3 99/03/30
 *
 * Copyright (c) 1997,1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.util;

import java.io.InputStream;
import java.io.IOException;
import java.util.Hashtable;
import pnuts.lang.PnutsException;

/**
 * A concrete ClassLoader.
 *
 * @version	1.3
 */
public class DefaultClassLoader extends ClassLoader {
    Hashtable classes = new Hashtable();

    /**
     * load <em>name</em> class.
     * If <em>resolve</em> is true resolveClass() is called.
     */
    public Class loadClass(String name, boolean resolve) throws ClassNotFoundException {
	Class cl = (Class)classes.get(name);
	if (cl == null) {
	    try {
		return findSystemClass(name);
	    } catch (Throwable e) {  } // ignore
	    cl = findClass(name);
	}
	if (cl == null) {
	    throw new ClassNotFoundException(name);
	}
	if (resolve) {
	    resolveClass(cl);
	}
	return cl;
    }

    private Class loadClass(InputStream in) throws IOException {
	try {
	    byte data[] = new byte[4096];
	    int total = 0, n;
                
	    while ((n = in.read(data, total, data.length - total)) >= 0) {
		if ((total += n) == data.length) {
		    byte newdata[] = new byte[total * 2];
		    System.arraycopy(data, 0, newdata, 0, total);
		    data = newdata;
		}
	    }
	    return defineClass(null, data, 0, total);
        } catch (Throwable e) {
            throw new PnutsException(e);
	} finally {
	    if (in != null) {
		in.close();
	    }
	}
    }

    void resolve(Class c) { 
	resolveClass(c);
    }

    /**
     * Read Bytecode <em>data</em> <em>length</em> bytes from <em>offset</em>
     * and define <em>name</em> class.
     */
    public Class define(String name, byte data[], int offset, int length){
	Class c = super.defineClass(name, data, offset, length);
	classes.put(c.getName(), c);
	resolve(c);
	return c;
    }

    /**
     * Find <em>name</em> class out of already loaded classes.
     */
    public Class findClass(String name) throws ClassNotFoundException {
	return super.findLoadedClass(name);
    }
}
