/*
 * @(#) EventAdapter.java	1.8 99/03/30
 *
 * Copyright (c) 1997,1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * See the file "license.txt" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */
package pnuts.util;

import java.awt.AWTEvent;
import java.awt.event.*;
import java.util.Hashtable;
import java.io.Serializable;
import java.io.PrintWriter;
import pnuts.lang.PnutsFunction;
import pnuts.lang.Context;
import pnuts.lang.Pnuts;

/**
 * An adapter class for AWT events.
 *
 * @version	1.8
 * @author	Toyokazu Tomatsu
 */
public class EventAdapter implements 
	ComponentListener, ContainerListener, FocusListener, KeyListener,
	MouseListener, MouseMotionListener, WindowListener,
	ActionListener, ItemListener, AdjustmentListener,
	TextListener, Serializable
{
    static Integer one = new Integer(1);

    /**
     * @serial
     */
    Hashtable ftable = new Hashtable();

    transient Context context;

    public EventAdapter(){
	this(new Context());
    }

    public EventAdapter(Context context){
	this.context = context;
    }

    public void bind(String eventType, PnutsFunction func){
	ftable.put(eventType, func);
    }

    public void unbind(String eventType, PnutsFunction func){
	ftable.remove(eventType);
    }

    public void dispatch(String eventType, AWTEvent e){
	PnutsFunction f = (PnutsFunction)ftable.get(eventType);
	if (f != null){
	    try {
		f.call(new Object[]{e}, context);
		context.getOutputStream().flush();
	    } catch (Throwable t){
		PrintWriter err = context.getErrorStream();
		if (Pnuts.isVerbose()){
		    t.printStackTrace(err);
		} else {
		    err.println(t);
		}
		err.flush();
	    }
	}
    }

    public void componentResized(ComponentEvent e) {
	dispatch("componentResized", e);
    }

    public void componentMoved(ComponentEvent e){
	dispatch("componentMoved", e);
    }

    public void componentShown(ComponentEvent e) {
	dispatch("componentShown", e);
    }

    public void componentHidden(ComponentEvent e) {
	dispatch("componentHidden", e);
    }

    public void componentAdded(ContainerEvent e) {
	dispatch("componentAdded", e);
    }

    public void componentRemoved(ContainerEvent e) {
	dispatch("componentRemoved", e);
    }

    public void focusGained(FocusEvent e) {
	dispatch("focusGained", e);
    }

    public void focusLost(FocusEvent e) {
	dispatch("focusLost", e);
    }

    public void keyTyped(KeyEvent e) {
	dispatch("keyTyped", e);
    }

    public void keyPressed(KeyEvent e) {
	dispatch("keyPressed", e);
    }

    public void keyReleased(KeyEvent e) {
	dispatch("keyReleased", e);
    }

    public void mouseClicked(MouseEvent e) {
	dispatch("mouseClicked", e);
    }

    public void mousePressed(MouseEvent e) {
	dispatch("mousePressed", e);
    }

    public void mouseReleased(MouseEvent e) {
	dispatch("mouseReleased", e);
    }

    public void mouseEntered(MouseEvent e) {
	dispatch("mouseEntered", e);
    }

    public void mouseExited(MouseEvent e) {
	dispatch("mouseExited", e);
    }

    public void mouseDragged(MouseEvent e) {
	dispatch("mouseDragged", e);
    }

    public void mouseMoved(MouseEvent e) {
	dispatch("mouseMoved", e);
    }

    public void windowOpened(WindowEvent e) {
	dispatch("windowOpened", e);
    }

    public void windowClosing(WindowEvent e) {
	dispatch("windowClosing", e);
    }

    public void windowClosed(WindowEvent e) {
	dispatch("windowClosed", e);
    }

    public void windowIconified(WindowEvent e) {
	dispatch("windowIconified", e);
    }

    public void windowDeiconified(WindowEvent e) {
	dispatch("windowDeiconified", e);
    }

    public void windowActivated(WindowEvent e) {
	dispatch("windowActivated", e);
    }

    public void windowDeactivated(WindowEvent e) {
	dispatch("windowDeactivated", e);
    }

    public void actionPerformed(ActionEvent e) {
	dispatch("actionPerformed", e);
    }

    public void itemStateChanged(ItemEvent e) {
	dispatch("itemStateChanged", e);
    }

    public void adjustmentValueChanged(AdjustmentEvent e) {
	dispatch("adjustmentValueChanged", e);
    }

    public void textValueChanged(TextEvent e) {
	dispatch("textValueChanged", e);
    }
}
